using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using GGJ_Engine;
using System.IO;

namespace GGJ_Game
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game_GGJ : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;

        private Interface startScreen;
        private Interface creditsScreen;
        private IngameHud ingameHud;

        // level data
        int levelIndex = -1;
        int numberOfLevels = 13;
        int numberOfCreditLevels = 6;
        Level level = null;
        float timeStayInGameState = 0.0f;        

        SpriteBatch spriteBatch;

        private enum GameState
        {
            StartScreen,
            Ingame,
            CreditsScreen
        }
        private GameState gameState = GameState.StartScreen;

        private enum Songs
        {
            BG_Music_0,
            BG_Music_1,
            BG_Music_2,
            Song_Count
        }

        private SoundEffect finishJingle;
        private SoundEffectInstance finishJingleInst;

        Song bg0Song;
        Song bg1Song;
        Song bg2Song;

        bool isNextLevelKeyReady = false;
        bool isLastLevelKeyReady = false;

        public Game_GGJ()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            finishJingle = Content.Load<SoundEffect>("sound/finish jingle");

            finishJingleInst = finishJingle.CreateInstance();
            finishJingleInst.IsLooped = false;
            finishJingleInst.Volume = 0.65f;

            bg0Song = Content.Load<Song>("sound/bg music 0");
            bg1Song = Content.Load<Song>("sound/bg music 1");
            bg2Song = Content.Load<Song>("sound/bg music 2");
        }

        public void playFinishJingle()
        {
            if (finishJingleInst != null)
            {
                finishJingleInst.Play();
            }
        }

        private void playMusic(Songs song)
        {
            try
            {
                if (song == Songs.BG_Music_0)
                {
                    if (bg0Song != null)
                    {
                        MediaPlayer.Volume = 1.0f;
                        MediaPlayer.Play(bg0Song);
                    }
                }
                else if (song == Songs.BG_Music_1)
                {
                    if (bg1Song != null)
                    {
                        MediaPlayer.Volume = 0.75f;
                        MediaPlayer.Play(bg1Song);
                    }
                }
                else if (song == Songs.BG_Music_2)
                {
                    if (bg2Song != null)
                    {
                        MediaPlayer.Volume = 0.75f;
                        MediaPlayer.Play(bg2Song);
                    }
                }
            }
            catch (Exception)
            {
            }
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            this.graphics.PreferredBackBufferWidth = 1280;
            this.graphics.PreferredBackBufferHeight = 1024;
            this.graphics.ApplyChanges();
            this.spriteBatch = new SpriteBatch(GraphicsDevice);

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // TODO: use this.Content to load your game content here
            startScreen = new ScreenInterface(GraphicsDevice, Content, "screens/intro");
            startScreen.LoadContent();

            creditsScreen = new ScreenInterface(GraphicsDevice, Content, "screens/credits");
            creditsScreen.LoadContent();

            ingameHud = new IngameHud(GraphicsDevice, Content);
            ingameHud.LoadContent();
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        private void LoadNextLevel()
        {
            // move to the next level
            levelIndex = levelIndex+1;

            // Unloads the content for the current level before loading the next one.
            if (level != null)
                level.Dispose();

            // Load the level.
            string levelPath = string.Format("Content/level/{0}.txt", levelIndex);

            if (levelIndex >= (this.numberOfLevels + this.numberOfCreditLevels))
            {
                // Last credits level was shown, switch to exit screen
                gameState = GameState.CreditsScreen;
                levelIndex = -1;
                return;
            }
            if (levelIndex >= this.numberOfLevels)
            {
                // End of level reached, show credits level
                levelPath = string.Format("Content/level/credits{0}.txt", levelIndex - this.numberOfLevels);
            }

            using (Stream fileStream = TitleContainer.OpenStream(levelPath))
                level = new Level(Services, fileStream, levelIndex, graphics);

            timeStayInGameState = 0.0f;
            level.Game = this;
            ingameHud = new IngameHud(graphics.GraphicsDevice, Content);
            ingameHud.LoadContent();
        }

        private void OnStartScreenExit(InterfaceElement sender)
        {
            gameState = GameState.Ingame;

            levelIndex = -1;
            LoadNextLevel();
            timeStayInGameState = 0.0f;
        }

        private void OnCreditsScreenExit(InterfaceElement sender)
        {
            // Restart the program
            gameState = GameState.StartScreen;
            timeStayInGameState = 0.0f;
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            if (!this.graphics.IsFullScreen)
            {
                //this.graphics.PreferredBackBufferWidth = this.GraphicsDevice.DisplayMode.Width;
                //this.graphics.PreferredBackBufferHeight = this.GraphicsDevice.DisplayMode.Height;
                //this.graphics.ToggleFullScreen();
            }

            // Allows the game to exit
            if (Keyboard.GetState().IsKeyDown(Keys.Escape))
                this.Exit();

            if (Keyboard.GetState().IsKeyDown(Keys.R))
            {
                this.levelIndex--;
                this.LoadNextLevel();
            }
            if (this.isNextLevelKeyReady && Keyboard.GetState().IsKeyDown(Keys.N))
            {
                this.LoadNextLevel();
                isNextLevelKeyReady = false;
            }
            if (this.isNextLevelKeyReady == false && Keyboard.GetState().IsKeyUp(Keys.N))
            {
                this.isNextLevelKeyReady = true;
            }
            if (this.isLastLevelKeyReady && Keyboard.GetState().IsKeyDown(Keys.B))
            {
                this.levelIndex--;
                this.levelIndex--;
                if (levelIndex < -1)
                    levelIndex = 0;
                this.LoadNextLevel();
                isLastLevelKeyReady = false;
            }
            if (this.isLastLevelKeyReady == false && Keyboard.GetState().IsKeyDown(Keys.B))
            {
                this.isLastLevelKeyReady = true; ;
            }
            
            if (this.isNextLevelKeyReady == false && Keyboard.GetState().IsKeyUp(Keys.N))
            {
                this.isNextLevelKeyReady = true;
            }

            switch (gameState)
            {
            case GameState.StartScreen:
                {
                    timeStayInGameState += (float)gameTime.ElapsedGameTime.TotalSeconds;
                    if (timeStayInGameState > 3.0f)
                    {
                        if (anyKeyPressed())
                        {
                            OnStartScreenExit(null);
                        }
                    }
                    break;
                }
            case GameState.Ingame:
                UpdateIngame(gameTime);
                if (level.HasReachedExit)
                {
                    timeStayInGameState += (float)gameTime.ElapsedGameTime.TotalSeconds;
                    if (timeStayInGameState > 3.0f)
                    {
                        LoadNextLevel();
                    }
                }
                break;
            case GameState.CreditsScreen:
                {
                    timeStayInGameState += (float)gameTime.ElapsedGameTime.TotalSeconds;
                    // Allow changing input after some seconds to avoid misclicking
                    if (timeStayInGameState > 3.0f)
                    {
                        if (anyKeyPressed())
                        {
                            OnCreditsScreenExit(null);
                        }
                    }
                    break;
                }
            }
            
            try
            {
                if (MediaPlayer.State == MediaState.Stopped || MediaPlayer.State == MediaState.Paused)
                {
                    Random rand = new Random(gameTime.TotalGameTime.Milliseconds);
                    int randomSongIndex = rand.Next((int)Songs.Song_Count);
                    playMusic((Songs)randomSongIndex);
                }
            }
            catch (Exception)
            {
            }

            base.Update(gameTime);
        }

        void UpdateIngame(GameTime gameTime)
        {
            ingameHud.Update();
            level.Update(gameTime, Keyboard.GetState());
            level.UpdateHud(ingameHud);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.BurlyWood);

            // Use gamestate to update/draw the right interface
            switch (gameState)
            {
                case GameState.StartScreen:
                    startScreen.Update();
                    startScreen.Draw();
                    break;
                case GameState.Ingame:
                    spriteBatch.Begin();
                    level.Draw(gameTime, spriteBatch);
                    spriteBatch.End();
                    ingameHud.Draw();
                    break;
                case GameState.CreditsScreen:
                    creditsScreen.Update();
                    creditsScreen.Draw();
                    break;
            }

            base.Draw(gameTime);
        }

        bool anyKeyPressed()
        {
            if (Keyboard.GetState().IsKeyDown(Keys.Space) || Keyboard.GetState().IsKeyDown(Keys.LeftControl) ||
                Keyboard.GetState().IsKeyDown(Keys.Up) || Keyboard.GetState().IsKeyDown(Keys.W) ||
                Keyboard.GetState().IsKeyDown(Keys.Right) || Keyboard.GetState().IsKeyDown(Keys.D) ||
                Keyboard.GetState().IsKeyDown(Keys.Down) || Keyboard.GetState().IsKeyDown(Keys.S) ||
                Keyboard.GetState().IsKeyDown(Keys.Left) || Keyboard.GetState().IsKeyDown(Keys.A))
            {
                return true;
            }
            return false;
        }
    }
}
