﻿using System;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Audio;

namespace GGJ_Game
{
    class Player
    {
        public Vector2 Position
        {
            get { return position; }
            set { position = value; }
        }
        Vector2 position;
        Vector2 floatingPosition;

        public Vector2 Velocity
        {
            get { return velocity; }
            set { velocity = value; }
        }
        Vector2 velocity;

        string dampingDebugString = "";

        public bool HasLegs
        {
            get { return hasLegs; }
        }
        bool hasLegs = false;
        public bool HasEyes
        {
            get { return hasEyes; }
        }
        bool hasEyes = false;
        public bool HasArms
        {
            get { return hasArms; }
        }
        bool hasArms = false;
        public bool HasBrain
        {
            get { return hasBrain; }
        }
        bool hasBrain = false;

        private void switchHasLegs(bool newHasLegs)
        {
            // Calculate bounds within texture size.    
            int width;
            int left;
            int height;
            int top;

            if (newHasLegs)
            {
                moveAccelleration = 6000;
                maxMovementSpeed = 200;
                groundFrictionFactor = 1.9f;
                linearDrag = 620;

                width = (int)(idleAnimation.FrameWidth() * 0.5);
                left = (idleAnimation.FrameWidth() - width) / 2;
                height = (int)(idleAnimation.FrameHeight());
                top = idleAnimation.FrameHeight() - height;
            }
            else
            {
                moveAccelleration = 3000;
                maxMovementSpeed = 100;
                groundFrictionFactor = 1.7f;
                linearDrag = 620;

                width = (int)(crawlIdleAnimation.FrameWidth() * 0.8);
                left = (crawlIdleAnimation.FrameWidth() - width) / 2;
                height = (int)(crawlIdleAnimation.FrameHeight() * 0.5);
                top = crawlIdleAnimation.FrameHeight() - height;
            }

            collisionOffset = new Rectangle(left, top, width, height);
            hasLegs = newHasLegs;
        }

        private void switchHasEyes(bool newHasEyes)
        {
            hasEyes = newHasEyes;
        }

        private void switchHasArms(bool newHasArms)
        {
            hasArms = newHasArms;
        }

        private void switchHasBrains(bool newHasBrains)
        {
            hasBrain = newHasBrains;
        }

        private SpriteSheet idleAnimation;
        private SpriteSheet idleArmAnimation;
        private SpriteSheet idleBrainAnimation;
        private SpriteSheet idleEyeAnimation;
        private SpriteSheet crawlAnimation;
        private SpriteSheet crawlArmAnimation;
        private SpriteSheet crawlEyeAnimation;
        private SpriteSheet crawlBrainAnimation;
        private SpriteSheet crawlAttackAnimation;
        private SpriteSheet crawlAttackEyeAnimation;
        private SpriteSheet crawlAttackBrainAnimation;
        private SpriteSheet crawlIdleAnimation;
        private SpriteSheet crawlIdleArmAnimation;
        private SpriteSheet crawlIdleEyeAnimation;
        private SpriteSheet crawlIdleBrainAnimation;
        private SpriteSheet runAnimation;
        private SpriteSheet runArmAnimation;
        private SpriteSheet runBrainAnimation;
        private SpriteSheet runEyeAnimation;
        private SpriteSheet jumpAnimation;
        private SpriteSheet jumpArmAnimation;
        private SpriteSheet jumpBrainAnimation;
        private SpriteSheet jumpEyeAnimation;
        private SpriteSheet meeleAttackAnimation;
        private SpriteSheet meeleAttackBrainAnimation;
        private SpriteSheet meeleAttackEyeAnimation;

        private AnimationPlayer animationPlayer;
        private AnimationPlayer animationPlayerArm;
        private AnimationPlayer animationPlayerBrain;
        private AnimationPlayer animationPlayerEye;

        // Sounds
        private SoundEffect jumpSound;
        private SoundEffect hitSound;
        private SoundEffect moveSound;
        private SoundEffect crawlSound;
        private SoundEffect itemPickupSound;

        private SoundEffectInstance jumpSoundInst;
        private SoundEffectInstance hitSoundInst;
        private SoundEffectInstance moveSoundInst;
        private SoundEffectInstance crawlSoundInst;
        private SoundEffectInstance itemPickupSoundInst;

        private const float GravityAcceleration = 3200;
        private const float airDragFactor = 0.6f;

        private float moveAccelleration;
        private float maxMovementSpeed;
        private float groundFrictionFactor;
        private float linearDrag;

        private SpriteFont hudFont;

        // Input
        private float movementDirection;
        private float movementAcceleration;
        private bool isAttackingKeyPressed;     // is the attacking key pressed
        private bool isJumping;                 // is current jumping
        private bool isFallingThroughGround;    // is current jumping

        // Jumping state
        private bool wasJumping;                // was jumping last frame
        private float jumpTime;
        // Vertical movement constants
        private const float MaxJumpTime = 0.35f;
        private const float JumpLaunchVelocity = -3600;
        private const float MaxFallSpeed = 500;
        private const float JumpControlPower = 0.14f;
        // Attacking constants
        private const float TimePerAttack = 0.5f;

        private float attackTime;
        private bool attackToRight;

        public bool IsAttacking
        {
            get { return isAttacking; }
            set { isAttacking = value; }
        }
        private bool isAttacking;
        
        // True if the players feet are on ground
        public bool IsOnGround
        {
            get { return isOnGround; }
        }
        bool isOnGround;

        Level level;

        public bool HasReachedExit
        {
            get { return hasReachedExit; }
        }
        bool hasReachedExit = false;

        // The collisionOffset is subtracted from the collision box, to have a smoother collision, not 100% on touch with the sprite
        private Rectangle collisionOffset;
        public Rectangle CollisionBox 
        {
            get {
                    int xPos = (int)Math.Round(position.X - animationPlayer.Orgin.X) + collisionOffset.X;
                    int yPos = (int)Math.Round(position.Y - animationPlayer.Orgin.Y) + collisionOffset.Y;

                    return new Rectangle(xPos, yPos, collisionOffset.Width, collisionOffset.Height);
                }
        }
        private float previousBottom;

        public Player(Level level, Vector2 position)
        {
            this.level = level;

            idleAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/idle"), 0.1f, true);
            idleArmAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/idle_arm"), 0.1f, true);
            idleBrainAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/idle_brain"), 0.1f, true);
            idleEyeAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/idle_eye"), 0.1f, true);
            crawlAttackAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/crawl_attack"), TimePerAttack / 6, false);
            crawlAttackEyeAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/crawl_attack_eye"), TimePerAttack / 6, false);
            crawlAttackBrainAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/crawl_attack_brain"), TimePerAttack / 6, false);
            crawlAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/crawl"), 0.1f, true);
            crawlArmAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/crawl_arm"), 0.1f, true);
            crawlEyeAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/crawl_eye"), 0.1f, true);
            crawlBrainAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/crawl_brain"), 0.1f, true);
            crawlIdleAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/crawl_idle"), 0.1f, true);
            crawlIdleArmAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/crawl_idle_arm"), 0.1f, true);
            crawlIdleEyeAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/crawl_idle_eye"), 0.1f, true);
            crawlIdleBrainAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/crawl_idle_brain"), 0.1f, true);
            runAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/run"), 0.1f, true);
            runArmAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/run_arm"), 0.1f, true);
            runBrainAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/run_brain"), 0.1f, true);
            runEyeAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/run_eye"), 0.1f, true);
            jumpAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/jump"), 0.1f, true);
            jumpArmAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/jump_arm"), 0.1f, true);
            jumpBrainAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/jump_brain"), 0.1f, true);
            jumpEyeAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/jump_eye"), 0.1f, true);
            meeleAttackAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/attack"), TimePerAttack / 9, false);
            meeleAttackBrainAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/attack_brain"), TimePerAttack / 9, false);
            meeleAttackEyeAnimation = new SpriteSheet(level.ContentManager.Load<Texture2D>("player/attack_eye"), TimePerAttack / 9, false);
            
            animationPlayer = new AnimationPlayer();
            animationPlayerArm = new AnimationPlayer();
            animationPlayerBrain = new AnimationPlayer();
            animationPlayerEye = new AnimationPlayer();

            // Sound
            jumpSound = level.ContentManager.Load<SoundEffect>("sound/jump");
            itemPickupSound = level.ContentManager.Load<SoundEffect>("sound/itemPickup");
            hitSound = level.ContentManager.Load<SoundEffect>("sound/hit");
            moveSound = level.ContentManager.Load<SoundEffect>("sound/move");
            crawlSound = level.ContentManager.Load<SoundEffect>("sound/crawl");

            jumpSoundInst = jumpSound.CreateInstance();
            jumpSoundInst.IsLooped = false;
            jumpSoundInst.Volume = 0.45f;

            itemPickupSoundInst = itemPickupSound.CreateInstance();
            itemPickupSoundInst.IsLooped = false;
            itemPickupSoundInst.Volume = 0.8f;

            hitSoundInst = hitSound.CreateInstance();
            hitSoundInst.IsLooped = false;
            hitSoundInst.Volume = 0.8f;

            moveSoundInst = moveSound.CreateInstance();
            moveSoundInst.IsLooped = true;
            moveSoundInst.Volume = 1.0f;

            crawlSoundInst = crawlSound.CreateInstance();
            crawlSoundInst.IsLooped = true;
            crawlSoundInst.Volume = 0.55f;

            // initialize
            Reset(position);
            hudFont = level.ContentManager.Load<SpriteFont>("font/SpriteFont1");
        }

        public void Update(GameTime gameTime, KeyboardState keyboardState)
        {
            HandleInput(keyboardState);

            if (hasReachedExit)
            {
                DragPlayerToExit();
            }

            ApplyPhysics(gameTime);

            HandleAttack(gameTime);

            animationPlayer.Update(gameTime);
            animationPlayerArm.Update(gameTime);
            animationPlayerBrain.Update(gameTime);
            animationPlayerEye.Update(gameTime);

            if (isAttacking)
            {
                if (hasLegs)
                {
                    animationPlayer.PlayAnimation(meeleAttackAnimation);
                    if (hasArms) animationPlayerArm.PlayAnimation(null);
                    if (hasBrain) animationPlayerBrain.PlayAnimation(meeleAttackBrainAnimation, animationPlayer);
                    if (hasEyes) animationPlayerEye.PlayAnimation(meeleAttackEyeAnimation, animationPlayer);
                }
                else
                {
                    animationPlayer.PlayAnimation(crawlAttackAnimation);
                    if (hasArms) animationPlayerArm.PlayAnimation(null);
                    if (hasBrain) animationPlayerBrain.PlayAnimation(crawlAttackBrainAnimation, animationPlayer);
                    if (hasEyes) animationPlayerEye.PlayAnimation(crawlAttackEyeAnimation, animationPlayer);
                }
            }
            else if (isOnGround)
            {
                if (Math.Abs(Velocity.X) > 0.02f)
                {
                    if (hasLegs)
                    {
                        animationPlayer.PlayAnimation(runAnimation);
                        if (hasArms) animationPlayerArm.PlayAnimation(runArmAnimation);
                        if (hasBrain) animationPlayerBrain.PlayAnimation(runBrainAnimation);
                        if (hasEyes) animationPlayerEye.PlayAnimation(runEyeAnimation);
                    }
                    else
                    {
                        animationPlayer.PlayAnimation(crawlAnimation);
                        if (hasArms) animationPlayerArm.PlayAnimation(crawlArmAnimation);
                        if (hasBrain) animationPlayerBrain.PlayAnimation(crawlBrainAnimation);
                        if (hasEyes) animationPlayerEye.PlayAnimation(crawlEyeAnimation);
                    }
                }
                else
                {
                    if (hasLegs)
                    {
                        animationPlayer.PlayAnimation(idleAnimation);
                        if (hasArms)  animationPlayerArm.PlayAnimation(idleArmAnimation);
                        if (hasBrain) animationPlayerBrain.PlayAnimation(idleBrainAnimation);
                        if (hasEyes)  animationPlayerEye.PlayAnimation(idleEyeAnimation);
                    }
                    else
                    {
                        animationPlayer.PlayAnimation(crawlIdleAnimation);
                        if (hasArms) animationPlayerArm.PlayAnimation(crawlIdleArmAnimation);
                        if (hasBrain) animationPlayerBrain.PlayAnimation(crawlIdleBrainAnimation);
                        if (hasEyes) animationPlayerEye.PlayAnimation(crawlIdleEyeAnimation);
                    }
                }
            }

            isJumping = false;
        }

        public void HandleInput(KeyboardState keyboardState)
        {
            movementAcceleration = 0;
            if (keyboardState.IsKeyDown(Keys.Left) || keyboardState.IsKeyDown(Keys.A))
            {
                movementAcceleration -= 1;
            }
            else if (keyboardState.IsKeyDown(Keys.Right) || keyboardState.IsKeyDown(Keys.D))
            {
                movementAcceleration += 1;
            }

            // Check if the player wants to attack
            isAttackingKeyPressed = keyboardState.IsKeyDown(Keys.LeftControl);

            // Check if the player wants to jump.
            isJumping = keyboardState.IsKeyDown(Keys.Space) ||
                keyboardState.IsKeyDown(Keys.Up) || keyboardState.IsKeyDown(Keys.W);

            if (keyboardState.IsKeyDown(Keys.Down) || keyboardState.IsKeyDown(Keys.S))
            {
                isFallingThroughGround = true;
            }
            else
            {
                isFallingThroughGround = false;
            }
        }

        //int skip = 0;
        public void ApplyPhysics(GameTime gameTime)
        {/*
            skip--;
            if (skip > 0)
            {
                return;
            }
            else
            {
                skip = 10;
            }
            */
            float elapsedTime = (float)gameTime.ElapsedGameTime.TotalSeconds;

            Vector2 oldPosition = floatingPosition;

            // acceleration the player
            float oldXVel = velocity.X;

            velocity.X += movementAcceleration * elapsedTime * moveAccelleration * groundFrictionFactor;
            velocity.Y = MathHelper.Clamp(velocity.Y + (elapsedTime * GravityAcceleration), -MaxFallSpeed, MaxFallSpeed);

            // Let player jump
            ApplyJump(gameTime);

            // Slowly lose speed
            double dampingExponent = -gameTime.ElapsedGameTime.TotalSeconds;
            if (isOnGround)
            {
                dampingExponent *= groundFrictionFactor;
            }
            else
                dampingExponent *= airDragFactor;
            float dampingFactor = (float)Math.Exp(dampingExponent);
            velocity.X = velocity.X * dampingFactor;

            if (velocity.X < 0) 
            {
                movementDirection = -1;
            }
            else if (velocity.X > 0)
            {
                movementDirection = 1;
            }

            if (velocity.X != 0)
            {
                if (hasLegs)
                {
                    if (moveSoundInst != null)
                        moveSoundInst.Play();
                    if (crawlSoundInst != null)
                        crawlSoundInst.Stop();
                }
                else
                {
                    if (crawlSoundInst != null)
                        crawlSoundInst.Play();
                    if (moveSoundInst != null)
                        moveSoundInst.Stop();
                }
            }
            else
            {
                if (moveSoundInst != null)
                    moveSoundInst.Stop();
                if (crawlSoundInst != null)
                    crawlSoundInst.Stop();
            }

            if (movementAcceleration == 0)
            {
                float currLinearDrag = linearDrag * elapsedTime * movementDirection;
                if (Math.Abs(currLinearDrag) >= Math.Abs(velocity.X))
                {
                    velocity.X = 0;
                }
                else
                {
                    velocity.X = velocity.X - currLinearDrag;
                }
            }

            velocity.X = MathHelper.Clamp(velocity.X, -maxMovementSpeed, maxMovementSpeed);


            dampingDebugString = (oldXVel - velocity.X).ToString();

            // Apply velocity
            floatingPosition += velocity * elapsedTime;
            Position = new Vector2((float)Math.Round(floatingPosition.X), (float)Math.Round(floatingPosition.Y));

            CheckCollision(Position);

            // Stop velocity after collision
            if (position.X == oldPosition.X)
            {
                velocity.X = 0;
            }
            if (position.Y == oldPosition.Y)
            {
                velocity.Y = 0;
            }
        }

        // Works on the velocity.x value. Use physics for jumping curve and gravity.
        // Can change velocity.x
        private void ApplyJump(GameTime gameTime)
        {
            if (hasLegs && isJumping)
            {
                // Begin or continue a jump
                if ((!wasJumping && isOnGround) || jumpTime > 0.0f)
                {
                    if (jumpTime == 0.0f)
                    {
                        // Play sound
                        jumpSoundInst.Play();
                    }

                    jumpTime += (float)gameTime.ElapsedGameTime.TotalSeconds;
                    animationPlayer.PlayAnimation(jumpAnimation);
                    if (hasArms) animationPlayerArm.PlayAnimation(jumpArmAnimation);
                    if (hasBrain) animationPlayerBrain.PlayAnimation(jumpBrainAnimation);
                    if (hasEyes) animationPlayerEye.PlayAnimation(jumpEyeAnimation);
                }

                if (0.0f < jumpTime && jumpTime <= MaxJumpTime)
                {
                    // Fully override the vertical velocity with a power curve that gives players more control over the top of the jump
                    velocity.Y = JumpLaunchVelocity * (1.0f - (float)Math.Pow(jumpTime / MaxJumpTime, JumpControlPower));
                }
                else
                {
                    // Reached the apex of the jump
                    jumpTime = 0.0f;
                }
            }
            else
            {
                jumpTime = 0.0f;
            }
            wasJumping = isJumping;
        }

        // Checks the collision and repositioning the player
        private void CheckCollision(Vector2 newPosition)
        {
            Rectangle bounds = CollisionBox;
            int leftTile = (int)Math.Floor((float)bounds.Left / level.tileSize.X);
            int rightTile = (int)Math.Ceiling((float)bounds.Right / level.tileSize.X) - 1;
            int topTile = (int)Math.Floor((float)bounds.Top / level.tileSize.Y);
            int bottomTile = (int)Math.Ceiling((float)bounds.Bottom / level.tileSize.Y) - 1;

            // Reset flag to search for ground collision.
            isOnGround = false;
            
            for (int x = leftTile; x <= rightTile; x++)
            {
                for (int y = topTile; y <= bottomTile; y++)
                {
                    // Check for collision
                    TileBlocking tileBlocking = level.getCollision(x, y);
                    if (tileBlocking != TileBlocking.Blocking_No)
                    {
                        // Determine collision depth (with direction) and magnitude.
                        Rectangle tileBounds = level.getBounds(x, y);
                        Vector2 depth = RectangleExtensions.GetIntersectionDepth(bounds, tileBounds);
                        if (depth != Vector2.Zero)
                        {
                            float absDepthX = Math.Abs(depth.X);
                            float absDepthY = Math.Abs(depth.Y);

                            if (absDepthY <= absDepthX)
                            {
                                // If we crossed the top of a tile, we are on the ground.
                                if (previousBottom <= tileBounds.Top && !(isFallingThroughGround && tileBlocking == TileBlocking.Blocking_Half))
                                {
                                    isOnGround = true;
                                }

                                // Ignore plattforms, unless we are on ground
                                if (tileBlocking == TileBlocking.Blocking_Full || isOnGround)
                                {
                                    // Resolve the collision along the Y axis.
                                    Position = new Vector2(position.X, position.Y + depth.Y);
                                    floatingPosition = Position;

                                    jumpTime = 0;

                                    // update the collision
                                    bounds = CollisionBox;
                                }
                            }
                            else if (tileBlocking == TileBlocking.Blocking_Full || tileBlocking == TileBlocking.Blocking_Box)  // ignore platforms
                            {
                                // Resolve the collision along the X axis.
                                Position = new Vector2(position.X + depth.X, position.Y);
                                floatingPosition = Position;

                                // update the collision
                                bounds = CollisionBox;
                            }
                        }
                    }

                    // Items
                    Collectable collectable = level.getCollisionWithCollectable(x, y);
                    if (collectable != null)
                    {
                        bool playPickupSound = false;
                        if (collectable.typeName == Level.LegName)
                        {
                            switchHasLegs(true);
                            playPickupSound = true;
                        }
                        if (collectable.typeName == Level.ArmName)
                        {
                            switchHasArms(true);
                            playPickupSound = true;
                        }
                        if (collectable.typeName == Level.EyeName)
                        {
                            switchHasEyes(true);
                            playPickupSound = true;
                        }
                        if (collectable.typeName == Level.BrainName)
                        {
                            switchHasBrains(true);
                            playPickupSound = true;
                        }
                        if (collectable.typeName == Level.Exit && hasBrain && hasArms && hasEyes && hasLegs)
                        {
                            hasReachedExit = true;
                            if (level.Game != null)
                                level.Game.playFinishJingle();
                        }
                        level.Collectables.Remove(collectable);

                        // Play Sound
                        if (playPickupSound && itemPickupSoundInst != null)
                            itemPickupSoundInst.Play();
                    }
                }
            }

            previousBottom = bounds.Bottom;
        }
        
        private void DragPlayerToExit()
        {
            float middleExit = level.tileSize.X / 2.0f;
            float exitPixelPosX = level.ExitObject.xPos * level.tileSize.X + middleExit;
            float reachExitThrashHold = 5;
            if (exitPixelPosX + reachExitThrashHold < this.position.X)
            {
                // Go left to the exit
                movementAcceleration = -1;
            }
            else if (exitPixelPosX - reachExitThrashHold > this.position.X)
            {
                // go right to the exit
                movementAcceleration = 1;
            }
            else
            {
                // Reach goal, stand still
                movementAcceleration = 0;
                velocity.X = 0;
            }
        }
        public void HandleAttack(GameTime gameTime)
        {
            if (isAttackingKeyPressed && hasArms)
            {
                // Start attacking
                if (attackTime == 0.0f)
                {
                    isAttacking = true;
                    attackToRight = movementDirection > 0;
                }
            }

            if (isAttacking)
            {
                // Continue Attacking
                attackTime += (float)gameTime.ElapsedGameTime.TotalSeconds;
            }

            if (attackTime >= TimePerAttack)
            {
                // End Attacking
                attackTime = 0;
                isAttacking = false;
                CheckHitForAttack();
            }
        }
        
        public void CheckHitForAttack()
        {
            Rectangle bounds = calcAttackRectangle();
            int leftTile = (int)Math.Floor((float)bounds.Left / level.tileSize.X);
            int rightTile = (int)Math.Ceiling((float)bounds.Right / level.tileSize.X) - 1;
            int topTile = (int)Math.Floor((float)bounds.Top / level.tileSize.Y);
            int bottomTile = (int)Math.Ceiling((float)bounds.Bottom / level.tileSize.Y) - 1;

            for (int x = leftTile; x <= rightTile; x++)
            {
                for (int y = topTile; y <= bottomTile; y++)
                {
                    // Check for collision
                    Collectable box = level.getCollisionWithBox(x, y);
                    if (box != null)
                    {
                        if (hitSoundInst != null)
                        {
                            hitSoundInst.Play();
                        }
                        level.BoxObjects.Remove(box);
                    }
                }
            }
        }

        private Rectangle calcAttackRectangle()
        {
            Rectangle weaponDrawPosition = CollisionBox;
            if (attackToRight)
            {
                weaponDrawPosition.X += (int)level.tileSize.X;
            }
            else
            {
                weaponDrawPosition.X -= (int)level.tileSize.X;
            }
            return weaponDrawPosition;
        }

        public void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            SpriteEffects effects = SpriteEffects.None;
            if (movementDirection < 0)
            {
                effects = SpriteEffects.FlipHorizontally;
            }
            animationPlayer.Draw(gameTime, spriteBatch, position, effects);
            animationPlayerArm.Draw(gameTime, spriteBatch, position, effects);
            animationPlayerBrain.Draw(gameTime, spriteBatch, position, effects);
            animationPlayerEye.Draw(gameTime, spriteBatch, position, effects);

            // Debug draw position
            //spriteBatch.DrawString(hudFont, position.X.ToString() + " /" + position.Y.ToString(), new Vector2(0, 0), Color.Black);
            //spriteBatch.DrawString(hudFont, velocity.X.ToString() + " /" + velocity.Y.ToString(), new Vector2(400, 0), Color.Black);
            //spriteBatch.DrawString(hudFont, "attacking: " + isAttacking.ToString(), new Vector2(0, 25), Color.Black);

            //spriteBatch.DrawString(hudFont, "FPS: " + (1 / (float)gameTime.ElapsedGameTime.TotalSeconds).ToString(), new Vector2(0, 25), Color.Black);
            //spriteBatch.DrawString(hudFont, "Damping: " + dampingDebugString, new Vector2(400, 25), Color.Black);
        }

        public void Reset(Vector2 position)
        {
            this.position = position;
            floatingPosition = position;
            velocity = Vector2.Zero;
            attackTime = 0;
            jumpTime = 0;
            switchHasLegs(false);
            switchHasEyes(false);
            switchHasArms(false);
            switchHasBrains(false);
            hasReachedExit = false;

            animationPlayer.PlayAnimation(crawlIdleAnimation);
            animationPlayerArm.PlayAnimation(null);
            animationPlayerBrain.PlayAnimation(null);
            animationPlayerEye.PlayAnimation(null);
        }
    }
}
